_CL Components_ lets you declare Drupal components that you can import and
render in your Drupal site. Other solutions, like
the [Components module](https://www.drupal.org/project/components), only concern
themselves with templates, and not components. But a **template is only a part
of a component**
. [![A component's structure](/files/ksnip_20220313-095710.png)](/files/ksnip_20220313-095710.png)

How to create a component
-------------------------

A component is any directory in your Drupal install that contains
a `metadata.json`. This folder must also contain at least a `my-component.twig`
template. It is advised to create a `css` and `js` directory for your
stylesheets and JS scripts. Learn more about creating a component
in [the documentation](https://git.drupalcode.org/project/cl_components/-/blob/1.x/docs/writing-components.md)
. You will need to let Drupal know where to start scanning for components (
including subdirectories). You can add as many locations as you need in the
settings
page. ![Component settings](/files/0372497880814de187b6bc95707cb402.png)

How to render a component
-------------------------

In Drupal there are two main ways to render some HTML, via a render array, and
via Twig.

### Using a render array

There is a special render element that allows you to render a component based
solely on the component ID and the component props. See this example on
rendering the `my-button` component with the label _Click Me_ using
the `hook_page_top`: 'cl\_component', '#component' => 'my-button', '#variant'
=> 'primary', '#context' => \[ 'text' => t('Click Me'), \], \]; ?> This will
take care of CSS, JS, HTML, and assets for you.

### Via Twig templates

`{% inject 'my-button' variant 'primary' with { text: 'Click Me', } %} {% endinject 'my-button' %}`

Composing "top-down" components
-------------------------------

One significant improvement that the `{% inject %}` tag provides is the ability
to wire the Drupal data more easily. **Traditionally**, when you are not using
this module, in order to map your templates, you need to start at the content
type template (for instance) and then at some point you render `{{ content }}`.
This is where the render pipeline enters the cognitive black hole, you lose the
thread. To find the thread again you need to leave the IDE to your browser to
inspect the HTML in search of template name suggestions for your
fields/sub-components. After that you create the new twig template, etc. **With
this module you still start at the content type template and you do everything
there, if you want.** That is because you will develop all your components
separately and wire the data to in the Drupal templates. Consider the following
example for a custom
block: [![Component from Twig](/files/ksnip_20220316-105222.png)](/files/ksnip_20220316-105222.png)
The `my-card` component could be provided by a contributed module, a custom
module for your component library, or in the same theme that
holds `block--bundle--block-cta.twig.html`. Note that the `my-card` component
supports children HTML and children components, _similar_ to **how React works**
. With this technique you can map all the field data in its place by using the
necessary sub-components. Thus, you don't have to find the sub-templates to map
them to their corresponding components. **TIP:** leverage
the [No Markup module](https://www.drupal.org/project/nomarkup) in order to
easily clean out wrapping field markup from the `{{ content.field_name }}` in
your templates. See [more info](https://www.drupal.org/project/nomarkup) on why
avoiding `{{ content.field_name|render|striptags }}`
and `{{ node.field_name[0].getValue }}` is important.

Build a component library!
--------------------------

Now that we have components it is time to build a component library. Thanks
to [CL Server](https://www.drupal.org/project/cl_server) this is super easy! I
recommend [Storybook](https://storybook.js.org), but others are great options as
well. [![Storybook integration](/files/ksnip_20220316-164333.png)](/files/ksnip_20220316-164333.png)
