<?php

namespace Drupal\cl_components\Plugin\Field\FieldFormatter;

use Drupal\cl_components\Plugin\Field\FieldType\StyleSelectorItem;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;

/**
 * Plugin implementation of the 'cl_components_style_selector_table' formatter.
 *
 * @FieldFormatter(
 *   id = "cl_components_style_selector_table",
 *   label = @Translation("Table"),
 *   field_types = {"cl_components_style_selector"}
 * )
 */
class StyleSelectorTableFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $header[] = '#';
    $header[] = $this->t('Module');
    $header[] = $this->t('Variant');

    $table = [
      '#type' => 'table',
      '#header' => $header,
    ];

    foreach ($items as $delta => $item) {
      if (!$item instanceof StyleSelectorItem) {
        continue;
      }
      $row = [];

      $row[]['#markup'] = $delta + 1;

      if ($item->module) {
        $allowed_values = $item->allowedModuleValues();
        $row[]['#markup'] = $allowed_values[$item->module];
      }
      else {
        $row[]['#markup'] = '';
      }
      $row[]['#markup'] = $item->variant ?? '';

      $table[$delta] = $row;
    }

    return [$table];
  }

}
